************************************************************************
*     ---------------------------------
      REAL FUNCTION COHCRSNEKART(E,IPAR,IC)
*     ---------------------------------
*
*     (Purpose)
*       CROSS SECTION / O16 NUCLEUS ( 10**(-38) CM**2 )
*
*     (Input)
*       E     : INCIDENT NEUTRINO ENERGY ( GEV )
*       IPAR  : INCIDENT NEUTRINO ID
*       IC    : INTERACTION MODE
*                1 : C.C.
*                0 : N.C.
*     (Output)
*       COHCRSNE :  CROSS SECTION / O16 NUCLEUS ( 10**(-38) CM**2 )
*      
*     (Creation Date and Author)
*       2007.01.08 ; G.Mitsuka
*       2007.11.05 ; G.Mitsuka -- support target nucleus besides 16O
*
*     (Comment)
*       Calculation is based on Phys. Rev. D 74, 054007 (2006)
*
************************************************************************

      IMPLICIT NONE
#include "necard.h"
      INTEGER IPAR,IC,I
      REAL E,WIDTH,EEND(2),SIGEND(2),SLOPE,ESTART,fact
      REAL SIG1CCE(0:100),SIG2CCE(0:90),SIG3CCE(0:90),SIG4CCE(0:90)
      REAL SIG1CCM(0:100),SIG2CCM(0:90),SIG3CCM(0:90),SIG4CCM(0:90)
      REAL SIG1CCT(0:100),SIG2CCT(0:90),SIG3CCT(0:90),SIG4CCT(0:90)
      REAL SIG1NC(0:100), SIG2NC(0:90), SIG3NC(0:90), SIG4NC(0:90)
      REAL S0CCE(50),S1CCE(50),S2CCE(50),S3CCE(40)
      REAL S4CCE(50),S5CCE(40),S6CCE(50),S7CCE(40)
      REAL S0CCM(50),S1CCM(50),S2CCM(50),S3CCM(40)
      REAL S4CCM(50),S5CCM(40),S6CCM(50),S7CCM(40)
      REAL S0CCT(50),S1CCT(50),S2CCT(50),S3CCT(40)
      REAL S4CCT(50),S5CCT(40),S6CCT(50),S7CCT(40)
      REAL S0NC(50), S1NC(50), S2NC(50), S3NC(40)
      REAL S4NC(50), S5NC(40), S6NC(50), S7NC(40)

C     Charged Current
C     Electron neutrino
      EQUIVALENCE (SIG2CCE(0),SIG1CCE(100))
      EQUIVALENCE (SIG3CCE(0),SIG2CCE(90))
      EQUIVALENCE (SIG4CCE(0),SIG3CCE(90))
      EQUIVALENCE (SIG1CCE(1), S0CCE(1))
      EQUIVALENCE (SIG1CCE(51),S1CCE(1))
      EQUIVALENCE (SIG2CCE(1), S2CCE(1))
      EQUIVALENCE (SIG2CCE(51),S3CCE(1))
      EQUIVALENCE (SIG3CCE(1), S4CCE(1))
      EQUIVALENCE (SIG3CCE(51),S5CCE(1))
      EQUIVALENCE (SIG4CCE(1), S6CCE(1))
      EQUIVALENCE (SIG4CCE(51),S7CCE(1))
C     Muon neutrino
      EQUIVALENCE (SIG2CCM(0),SIG1CCM(100))
      EQUIVALENCE (SIG3CCM(0),SIG2CCM(90))
      EQUIVALENCE (SIG4CCM(0),SIG3CCM(90))
      EQUIVALENCE (SIG1CCM(1), S0CCM(1))
      EQUIVALENCE (SIG1CCM(51),S1CCM(1))
      EQUIVALENCE (SIG2CCM(1), S2CCM(1))
      EQUIVALENCE (SIG2CCM(51),S3CCM(1))
      EQUIVALENCE (SIG3CCM(1), S4CCM(1))
      EQUIVALENCE (SIG3CCM(51),S5CCM(1))
      EQUIVALENCE (SIG4CCM(1), S6CCM(1))
      EQUIVALENCE (SIG4CCM(51),S7CCM(1))
C     Tau neutrino
      EQUIVALENCE (SIG2CCT(0),SIG1CCT(100))
      EQUIVALENCE (SIG3CCT(0),SIG2CCT(90))
      EQUIVALENCE (SIG4CCT(0),SIG3CCT(90))
      EQUIVALENCE (SIG1CCT(1), S0CCT(1))
      EQUIVALENCE (SIG1CCT(51),S1CCT(1))
      EQUIVALENCE (SIG2CCT(1), S2CCT(1))
      EQUIVALENCE (SIG2CCT(51),S3CCT(1))
      EQUIVALENCE (SIG3CCT(1), S4CCT(1))
      EQUIVALENCE (SIG3CCT(51),S5CCT(1))
      EQUIVALENCE (SIG4CCT(1), S6CCT(1))
      EQUIVALENCE (SIG4CCT(51),S7CCT(1))

C     Neutral Current
C     (neutral current does not distinguish neutrino ID)
      EQUIVALENCE (SIG2NC(0),SIG1NC(100))
      EQUIVALENCE (SIG3NC(0),SIG2NC(90))
      EQUIVALENCE (SIG4NC(0),SIG3NC(90))
      EQUIVALENCE (SIG1NC(1), S0NC(1))
      EQUIVALENCE (SIG1NC(51),S1NC(1))
      EQUIVALENCE (SIG2NC(1), S2NC(1))
      EQUIVALENCE (SIG2NC(51),S3NC(1))
      EQUIVALENCE (SIG3NC(1), S4NC(1))
      EQUIVALENCE (SIG3NC(51),S5NC(1))
      EQUIVALENCE (SIG4NC(1), S6NC(1))
      EQUIVALENCE (SIG4NC(51),S7NC(1))

C     Table below assumes 12C nuclei, for other nuclei     
C     ScalingFactor:(msss#/12)^(1/3) should be applied
      real ScalingFactor

C     ===============
C     Charged current
C     ===============
C     Electron neutrino
C
C     0 - 1GeV

      DATA S0CCE/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000104, 0.0000279, 0.0000561, 0.0000965,
     &     0.0001485, 0.0002109, 0.0002875, 0.0003755, 0.0004715,
     &     0.0005831, 0.0007035, 0.0008377, 0.0009839, 0.0011449,
     &     0.0013192, 0.0015050, 0.0017086, 0.0019196, 0.0021381,
     &     0.0023707, 0.0026120, 0.0028608, 0.0031134, 0.0033792,
     &     0.0036532, 0.0039304, 0.0042152, 0.0045060, 0.0047948,
     &     0.0050944, 0.0053877, 0.0056975, 0.0060001, 0.0063197/
      DATA S1CCE/0.0066305, 0.0069441, 0.0072863, 0.0076040, 0.0079379,
     &     0.0082834, 0.0086159, 0.0089693, 0.0093349, 0.0096663,
     &     0.0100344, 0.0103930, 0.0107911, 0.0111466, 0.0115274,
     &     0.0119256, 0.0123274, 0.0127078, 0.0130909, 0.0135618,
     &     0.0139659, 0.0143311, 0.0147789, 0.0152237, 0.0156208,
     &     0.0160729, 0.0165280, 0.0169968, 0.0174111, 0.0178828,
     &     0.0183710, 0.0188292, 0.0192756, 0.0197936, 0.0202895,
     &     0.0207489, 0.0212350, 0.0218046, 0.0223101, 0.0227753,
     &     0.0232925, 0.0238053, 0.0243516, 0.0248955, 0.0254259,
     &     0.0259550, 0.0264983, 0.0270511, 0.0276330, 0.0281989/
      
C     1 - 10GeV
      
      DATA  S2CCE/0.0287555, 0.0347180, 0.0412164, 0.0482341, 0.0556889,
     &     0.0635720, 0.0718286, 0.0801317, 0.0886974, 0.0971809,
     &     0.1057323, 0.1141601, 0.1227186, 0.1311716, 0.1393352,
     &     0.1474732, 0.1556295, 0.1635293, 0.1713530, 0.1790138,
     &     0.1867290, 0.1942656, 0.2015527, 0.2087278, 0.2157515,
     &     0.2228392, 0.2296889, 0.2366020, 0.2432032, 0.2497309,
     &     0.2561987, 0.2626273, 0.2687329, 0.2749022, 0.2810517,
     &     0.2869722, 0.2927156, 0.2985199, 0.3043763, 0.3100655,
     &     0.3155749, 0.3211514, 0.3265048, 0.3318411, 0.3371385,
     &     0.3424042, 0.3473746, 0.3523890, 0.3570444, 0.3619509/
      DATA S3CCE/0.3668292, 0.3716234, 0.3763984, 0.3812423, 0.3857763,
     &     0.3903470, 0.3948632, 0.3991700, 0.4036645, 0.4080337,
     &     0.4123315, 0.4165214, 0.4205598, 0.4248032, 0.4287207,
     &     0.4327121, 0.4367357, 0.4407955, 0.4446641, 0.4484937,
     &     0.4522652, 0.4559930, 0.4599171, 0.4636951, 0.4674988,
     &     0.4711328, 0.4745877, 0.4782162, 0.4818023, 0.4852371,
     &     0.4888074, 0.4922886, 0.4957025, 0.4990975, 0.5024011,
     &     0.5055990, 0.5089697, 0.5121351, 0.5153198, 0.5185581/

C     10 - 100GeV
      
      DATA S4CCE/0.5218262, 0.5519786, 0.5798797, 0.6054676, 0.6299553,
     &     0.6526237, 0.6738511, 0.6940537, 0.7127109, 0.7309447,
     &     0.7480437, 0.7648663, 0.7804289, 0.7954702, 0.8098114,
     &     0.8241115, 0.8375118, 0.8505401, 0.8625031, 0.8748027,
     &     0.8861911, 0.8974295, 0.9083648, 0.9191304, 0.9292240,
     &     0.9392434, 0.9488909, 0.9584491, 0.9678700, 0.9768023,
     &     0.9850009, 0.9935705, 1.0023058, 1.0105563, 1.0184298,
     &     1.0261346, 1.0342007, 1.0417846, 1.0486129, 1.0556575,
     &     1.0630771, 1.0698116, 1.0761721, 1.0832148, 1.0897710,
     &     1.0964352, 1.1032423, 1.1095770, 1.1150956, 1.1214161/
      DATA S5CCE/1.1276763, 1.1330964, 1.1388295, 1.1442776, 1.1497573,
     &     1.1548576, 1.1606603, 1.1657616, 1.1709557, 1.1760648,
     &     1.1815160, 1.1859906, 1.1914043, 1.1960856, 1.2012735,
     &     1.2056885, 1.2100892, 1.2149408, 1.2195788, 1.2235955,
     &     1.2284277, 1.2329261, 1.2371358, 1.2415865, 1.2452972,
     &     1.2495234, 1.2539730, 1.2576986, 1.2617099, 1.2656336,
     &     1.2690484, 1.2736260, 1.2771265, 1.2808241, 1.2844432,
     &     1.2886701, 1.2927292, 1.2967596, 1.2998952, 1.3031765/
      
C     100 - 1000GeV
      
      DATA S6CCE/1.3064690, 1.3400396, 1.3715674, 1.3995383, 1.4262815,
     &     1.4488187, 1.4734295, 1.4951492, 1.5141051, 1.5337037,
     &     1.5516399, 1.5704030, 1.5871099, 1.6024107, 1.6199581,
     &     1.6330096, 1.6456158, 1.6596435, 1.6726407, 1.6852094,
     &     1.6966054, 1.7095306, 1.7206516, 1.7302761, 1.7403167,
     &     1.7503796, 1.7623786, 1.7702407, 1.7784233, 1.7900237,
     &     1.7976585, 1.8074283, 1.8140180, 1.8239368, 1.8299341,
     &     1.8400526, 1.8480108, 1.8529344, 1.8614008, 1.8684299,
     &     1.8758951, 1.8850197, 1.8899740, 1.8995238, 1.9042235,
     &     1.9092238, 1.9174260, 1.9225791, 1.9281225, 1.9339096/
      DATA S7CCE/1.9407427, 1.9486378, 1.9519287, 1.9584236, 1.9631920,
     &     1.9719222, 1.9765598, 1.9788380, 1.9885632, 1.9935441,
     &     1.9957797, 2.0015724, 2.0071225, 2.0066593, 2.0150243,
     &     2.0136952, 2.0225669, 2.0257400, 2.0318437, 2.0385451,
     &     2.0434118, 2.0435668, 2.0448400, 2.0575719, 2.0539905,
     &     2.0670929, 2.0677402, 2.0694691, 2.0716802, 2.0809571,
     &     2.0848104, 2.0857212, 2.0882557, 2.0959193, 2.1036749,
     &     2.1078637, 2.1111392, 2.1055301, 2.1159755, 2.1148912/
      
C     Muon neutrino
C
C     0 - 1GeV
      
      DATA S0CCM/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000/
      DATA S1CCM/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000587, 0.0001488, 0.0002567, 0.0003796, 0.0005165,
     &     0.0006637, 0.0008289, 0.0010009, 0.0011761, 0.0013437,
     &     0.0015514, 0.0017459, 0.0019555, 0.0021625, 0.0024013,
     &     0.0026281, 0.0028431, 0.0031042, 0.0033648, 0.0036046,
     &     0.0038659, 0.0041340, 0.0044113, 0.0046966, 0.0049871,
     &     0.0052745, 0.0055626, 0.0058793, 0.0061836, 0.0065265,
     &     0.0068437, 0.0071601, 0.0074974, 0.0078215, 0.0081516,
     &     0.0085141, 0.0088750, 0.0092689, 0.0096412, 0.0100191,
     &     0.0104098, 0.0107864, 0.0111524, 0.0115550, 0.0119650/

C     1 - 10GeV
      
      DATA  S2CCM/0.0123868, 0.0168220, 0.0218608, 0.0273692, 0.0335245,
     &     0.0400864, 0.0471618, 0.0545408, 0.0620953, 0.0697320,
     &     0.0775176, 0.0852003, 0.0930186, 0.1008002, 0.1084256,
     &     0.1160809, 0.1236129, 0.1311507, 0.1384696, 0.1458813,
     &     0.1528888, 0.1600905, 0.1670813, 0.1740926, 0.1808108,
     &     0.1875428, 0.1940960, 0.2006096, 0.2069742, 0.2133885,
     &     0.2196165, 0.2257806, 0.2317305, 0.2376828, 0.2436233,
     &     0.2494889, 0.2552479, 0.2607383, 0.2661822, 0.2716053,
     &     0.2770503, 0.2824260, 0.2876452, 0.2927830, 0.2978516,
     &     0.3029938, 0.3080093, 0.3129170, 0.3176703, 0.3224988/
      DATA S3CCM/0.3271923, 0.3318858, 0.3364420, 0.3409185, 0.3455037,
     &     0.3500564, 0.3544241, 0.3586626, 0.3629008, 0.3672373,
     &     0.3714762, 0.3756223, 0.3795874, 0.3837285, 0.3877976,
     &     0.3918183, 0.3956355, 0.3995780, 0.4034073, 0.4071924,
     &     0.4108496, 0.4146299, 0.4184007, 0.4221203, 0.4257334,
     &     0.4292427, 0.4327574, 0.4364043, 0.4398354, 0.4432727,
     &     0.4467237, 0.4500549, 0.4533566, 0.4566575, 0.4599649,
     &     0.4632397, 0.4664794, 0.4696469, 0.4727957, 0.4759566/

C     10 - 100GeV
      
      DATA S4CCM/0.4791157, 0.5090574, 0.5368219, 0.5621448, 0.5859807,
     &     0.6087536, 0.6295228, 0.6495585, 0.6685315, 0.6866756,
     &     0.7038337, 0.7200977, 0.7356420, 0.7505845, 0.7648598,
     &     0.7788678, 0.7919731, 0.8049135, 0.8172600, 0.8293833,
     &     0.8409622, 0.8519925, 0.8629092, 0.8735696, 0.8839752,
     &     0.8939421, 0.9034110, 0.9127921, 0.9222102, 0.9312880,
     &     0.9397278, 0.9484261, 0.9567917, 0.9649916, 0.9730241,
     &     0.9809406, 0.9884679, 0.9960305, 1.0034532, 1.0103180,
     &     1.0171712, 1.0241921, 1.0310835, 1.0377328, 1.0441240,
     &     1.0504821, 1.0569329, 1.0631651, 1.0693101, 1.0753080/
      DATA S5CCM/1.0813604, 1.0870122, 1.0926904, 1.0978417, 1.1034229,
     &     1.1088890, 1.1142087, 1.1195039, 1.1249382, 1.1300273,
     &     1.1350417, 1.1402506, 1.1450584, 1.1498971, 1.1547612,
     &     1.1594726, 1.1643064, 1.1690878, 1.1734693, 1.1780222,
     &     1.1821005, 1.1865996, 1.1909119, 1.1952087, 1.1992570,
     &     1.2035772, 1.2073471, 1.2113632, 1.2154314, 1.2194961,
     &     1.2234314, 1.2274832, 1.2313849, 1.2351427, 1.2389728,
     &     1.2428611, 1.2465391, 1.2502808, 1.2537293, 1.2574028/

C     100 - 1000GeV
      
      DATA S6CCM/1.2609625, 1.2948177, 1.3252962, 1.3537510, 1.3799519,
     &     1.4040847, 1.4273874, 1.4491001, 1.4696056, 1.4889720,
     &     1.5067012, 1.5239422, 1.5403472, 1.5562616, 1.5719667,
     &     1.5859296, 1.5998881, 1.6134733, 1.6259597, 1.6391338,
     &     1.6513508, 1.6623483, 1.6733082, 1.6847426, 1.6957094,
     &     1.7051853, 1.7161323, 1.7260731, 1.7355928, 1.7447662,
     &     1.7532912, 1.7626160, 1.7720537, 1.7796863, 1.7869441,
     &     1.7952801, 1.8039456, 1.8106983, 1.8176835, 1.8255082,
     &     1.8331841, 1.8406237, 1.8467836, 1.8547037, 1.8606763,
     &     1.8677570, 1.8733933, 1.8805575, 1.8850287, 1.8929435/
      DATA S7CCM/1.8969660, 1.9047528, 1.9102873, 1.9158197, 1.9220116,
     &     1.9272201, 1.9318493, 1.9387791, 1.9439635, 1.9483505,
     &     1.9547427, 1.9579969, 1.9644362, 1.9699360, 1.9731757,
     &     1.9783397, 1.9827844, 1.9882622, 1.9910886, 1.9985443,
     &     2.0024503, 2.0059155, 2.0098843, 2.0155855, 2.0193423,
     &     2.0237852, 2.0279367, 2.0320408, 2.0368848, 2.0396161,
     &     2.0438734, 2.0487962, 2.0497062, 2.0490910, 2.0594722,
     &     2.0629224, 2.0671886, 2.0697495, 2.0734579, 2.0777314/

C     Tau neutrino
C
C     0 - 1GeV

      DATA S0CCT/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000/
      DATA S1CCT/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000/
      
C     1 - 10GeV
      
      DATA  S2CCT/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000/
      DATA S3CCT/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000/
      
C     10 - 100GeV
      
      DATA S4CCT/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000443, 0.0002421, 0.0006375, 0.0012466, 0.0020777,
     &     0.0031386, 0.0043963, 0.0058614, 0.0075000, 0.0092739,
     &     0.0112384, 0.0133415, 0.0156098, 0.0180043, 0.0204522,
     &     0.0229174, 0.0256492, 0.0283369, 0.0311778, 0.0340174,
     &     0.0369294, 0.0399123, 0.0428154, 0.0458257, 0.0489551,
     &     0.0520267, 0.0551105, 0.0580594, 0.0614329, 0.0646356,
     &     0.0677241, 0.0708512, 0.0739551, 0.0772689, 0.0804403,
     &     0.0835954, 0.0867238, 0.0899625, 0.0932228, 0.0962483,
     &     0.0992503, 0.1024098, 0.1055827, 0.1088352, 0.1118402/
      DATA S5CCT/0.1149971, 0.1179565, 0.1211874, 0.1245042, 0.1274864,
     &     0.1304585, 0.1333726, 0.1362075, 0.1391089, 0.1422028,
     &     0.1452372, 0.1480157, 0.1509284, 0.1537896, 0.1567624,
     &     0.1595308, 0.1624148, 0.1652144, 0.1681782, 0.1710323,
     &     0.1737155, 0.1764644, 0.1790230, 0.1819154, 0.1845170,
     &     0.1872746, 0.1898531, 0.1926602, 0.1951881, 0.1981122,
     &     0.2006416, 0.2033822, 0.2059074, 0.2086767, 0.2111803,
     &     0.2136809, 0.2163029, 0.2187902, 0.2212357, 0.2235744/
      
C     100 - 1000GeV
      
      DATA S6CCT/0.2261804, 0.2497035, 0.2720966, 0.2932415, 0.3133605,
     &     0.3328188, 0.3511162, 0.3679929, 0.3847006, 0.4004301,
     &     0.4161230, 0.4310502, 0.4451320, 0.4585491, 0.4718029,
     &     0.4836676, 0.4960683, 0.5078645, 0.5194580, 0.5305713,
     &     0.5412863, 0.5516004, 0.5612694, 0.5713224, 0.5806891,
     &     0.5897869, 0.5993362, 0.6080056, 0.6166725, 0.6258581,
     &     0.6338975, 0.6422924, 0.6498261, 0.6575516, 0.6653477,
     &     0.6721635, 0.6793111, 0.6866748, 0.6934089, 0.7004224,
     &     0.7077530, 0.7141335, 0.7201734, 0.7269880, 0.7331561,
     &     0.7401467, 0.7457191, 0.7510905, 0.7568001, 0.7623860/
      DATA S7CCT/0.7685351, 0.7743082, 0.7798297, 0.7854649, 0.7908173,
     &     0.7964615, 0.8011621, 0.8064752, 0.8108025, 0.8157858,
     &     0.8207272, 0.8253815, 0.8301651, 0.8358346, 0.8397226,
     &     0.8448838, 0.8488096, 0.8532555, 0.8584445, 0.8624130,
     &     0.8666518, 0.8702722, 0.8750861, 0.8803684, 0.8835880,
     &     0.8876395, 0.8921728, 0.8965830, 0.9001207, 0.9040621,
     &     0.9078165, 0.9120809, 0.9146955, 0.9196000, 0.9231375,
     &     0.9267841, 0.9303812, 0.9346823, 0.9376741, 0.9399602/
      
C
C     Neutral current
C     ===============
C     0 - 1GeV
      
      DATA S0NC/0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
     &     0.0000000, 0.0000106, 0.0000234, 0.0000425, 0.0000684,
     &     0.0001001, 0.0001379, 0.0001826, 0.0002336, 0.0002901,
     &     0.0003534, 0.0004231, 0.0004982, 0.0005796, 0.0006688,
     &     0.0007652, 0.0008685, 0.0009792, 0.0010957, 0.0012162,
     &     0.0013441, 0.0014758, 0.0016114, 0.0017495, 0.0018933,
     &     0.0020397, 0.0021878, 0.0023423, 0.0025033, 0.0026594,
     &     0.0028198, 0.0029818, 0.0031433, 0.0033100, 0.0034745/
      DATA S1NC/0.0036490, 0.0038185, 0.0039926, 0.0041704, 0.0043486,
     &     0.0045342, 0.0047126, 0.0049019, 0.0050892, 0.0052847,
     &     0.0054718, 0.0056621, 0.0058724, 0.0060767, 0.0062782,
     &     0.0064758, 0.0066934, 0.0069084, 0.0071051, 0.0073348,
     &     0.0075523, 0.0077664, 0.0080080, 0.0082361, 0.0084388,
     &     0.0086771, 0.0089379, 0.0091861, 0.0093947, 0.0096535,
     &     0.0098996, 0.0101502, 0.0104011, 0.0106608, 0.0109247,
     &     0.0111896, 0.0114338, 0.0117108, 0.0119838, 0.0122602,
     &     0.0125332, 0.0127885, 0.0130842, 0.0133594, 0.0136580,
     &     0.0139586, 0.0142183, 0.0144949, 0.0148201, 0.0151221/
      
C     1 - 10GeV
      
      DATA S2NC/0.0154251, 0.0185826, 0.0220371, 0.0257400, 0.0296963,
     &     0.0338303, 0.0382676, 0.0426091, 0.0471697, 0.0516216,
     &     0.0561423, 0.0606360, 0.0651255, 0.0695745, 0.0738839,
     &     0.0782436, 0.0825038, 0.0867150, 0.0908022, 0.0949144,
     &     0.0989191, 0.1029010, 0.1067571, 0.1105816, 0.1142507,
     &     0.1179837, 0.1216375, 0.1252244, 0.1288110, 0.1322544,
     &     0.1355959, 0.1389851, 0.1422140, 0.1454440, 0.1487170,
     &     0.1519066, 0.1549271, 0.1579609, 0.1609614, 0.1640255,
     &     0.1669386, 0.1698824, 0.1726865, 0.1755206, 0.1783110,
     &     0.1810920, 0.1837174, 0.1864088, 0.1889650, 0.1913759/
      DATA S3NC/0.1939455, 0.1964975, 0.1990135, 0.2015162, 0.2038921,
     &     0.2062952, 0.2087189, 0.2110676, 0.2133902, 0.2156939,
     &     0.2179843, 0.2201902, 0.2224218, 0.2244844, 0.2266867,
     &     0.2287949, 0.2308652, 0.2329943, 0.2350789, 0.2370739,
     &     0.2390181, 0.2410144, 0.2430555, 0.2450774, 0.2470632,
     &     0.2489729, 0.2508138, 0.2527214, 0.2546102, 0.2564401,
     &     0.2582496, 0.2600645, 0.2618810, 0.2637733, 0.2655605,
     &     0.2671571, 0.2689263, 0.2706157, 0.2722989, 0.2740049/
      
C     10 - 100GeV
      
      DATA S4NC/0.2756961, 0.2916262, 0.3063510, 0.3198178, 0.3326301,
     &     0.3446779, 0.3558728, 0.3665745, 0.3765379, 0.3859180,
     &     0.3950384, 0.4038445, 0.4120437, 0.4199291, 0.4275618,
     &     0.4349445, 0.4421532, 0.4490458, 0.4555309, 0.4617403,
     &     0.4678788, 0.4738339, 0.4795256, 0.4851776, 0.4905321,
     &     0.4957902, 0.5008476, 0.5058758, 0.5108874, 0.5156166,
     &     0.5201606, 0.5245797, 0.5290202, 0.5333998, 0.5376686,
     &     0.5417969, 0.5458774, 0.5498790, 0.5536576, 0.5575055,
     &     0.5612629, 0.5646203, 0.5681511, 0.5716978, 0.5751848,
     &     0.5785789, 0.5819441, 0.5852580, 0.5887019, 0.5918938/
      DATA S5NC/0.5950404, 0.5981256, 0.6009673, 0.6039644, 0.6069155,
     &     0.6099091, 0.6126461, 0.6154675, 0.6182515, 0.6209053,
     &     0.6233637, 0.6260248, 0.6286493, 0.6313802, 0.6339369,
     &     0.6362349, 0.6387439, 0.6412004, 0.6436855, 0.6459650,
     &     0.6482495, 0.6505863, 0.6528956, 0.6551779, 0.6573146,
     &     0.6595424, 0.6617504, 0.6639281, 0.6658459, 0.6679731,
     &     0.6700768, 0.6718754, 0.6739965, 0.6760329, 0.6780473,
     &     0.6800742, 0.6820492, 0.6840036, 0.6859349, 0.6877674/
      
C     100 - 1000GeV
      
      DATA S6NC/0.6896631, 0.7075813, 0.7234233, 0.7385500, 0.7523960,
     &     0.7655222, 0.7776426, 0.7891208, 0.7998340, 0.8100088,
     &     0.8197618, 0.8289600, 0.8377687, 0.8465977, 0.8545737,
     &     0.8620425, 0.8694415, 0.8766398, 0.8831007, 0.8898504,
     &     0.8959618, 0.9022892, 0.9080289, 0.9138844, 0.9194558,
     &     0.9249806, 0.9299769, 0.9348471, 0.9399436, 0.9448810,
     &     0.9496118, 0.9539356, 0.9587472, 0.9632198, 0.9676079,
     &     0.9718467, 0.9751604, 0.9792389, 0.9832331, 0.9870651,
     &     0.9908971, 0.9947615, 0.9984460, 1.0020608, 1.0056077,
     &     1.0089174, 1.0123342, 1.0156904, 1.0189889, 1.0222301/
      DATA S7NC/1.0254664, 1.0284900, 1.0315732, 1.0346063, 1.0375919,
     &     1.0405317, 1.0433830, 1.0462337, 1.0490422, 1.0518092,
     &     1.0545359, 1.0571187, 1.0597698, 1.0625177, 1.0650964,
     &     1.0676402, 1.0699155, 1.0723893, 1.0748310, 1.0772415,
     &     1.0796216, 1.0820366, 1.0843577, 1.0865133, 1.0887707,
     &     1.0910080, 1.0932192, 1.0954085, 1.0975205, 1.0996552,
     &     1.1017666, 1.1038537, 1.1062649, 1.1081809, 1.1102001,
     &     1.1121978, 1.1141744, 1.1161304, 1.1177792, 1.1196917/
      
      SIG1CCE(0) = 0.
      SIG1CCM(0) = 0.
      SIG1CCT(0) = 0.
      SIG1NC(0) = 0.

C     
C	DETERMINE COHCRS VALUE
C

C	ENERGY FROM 0GEV 1GEV  ( 0.01GEV/BIN )

      IF (E.LT.1.0) THEN
         WIDTH = 0.01
         I = INT(E/WIDTH)
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG1CCE(I)
               SIGEND(2) = SIG1CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG1CCM(I)
               SIGEND(2) = SIG1CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG1CCT(I)
               SIGEND(2) = SIG1CCT(I+1)
            endif
         else
            SIGEND(1) = SIG1NC(I)
            SIGEND(2) = SIG1NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH
         EEND(2) = FLOAT(I+1)*WIDTH

C	ENERGY FROM 1GEV TO 10GEV  ( 0.1GEV/BIN )

      ELSE IF (E.LT.10.0) THEN
         ESTART = 1.
         WIDTH = 0.1
         I = INT((E-ESTART)/WIDTH)
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG2CCE(I)
               SIGEND(2) = SIG2CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG2CCM(I)
               SIGEND(2) = SIG2CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG2CCT(I)
               SIGEND(2) = SIG2CCT(I+1)
            endif
         else
            SIGEND(1) = SIG2NC(I)
            SIGEND(2) = SIG2NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C	ENERGY FROM 10GEV TO 100GEV  ( 1GEV/BIN )

      ELSE IF (E.LT.100.0) THEN
         ESTART = 10.
         WIDTH = 1.
         I = INT((E-ESTART)/WIDTH)
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG3CCE(I)
               SIGEND(2) = SIG3CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG3CCM(I)
               SIGEND(2) = SIG3CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG3CCT(I)
               SIGEND(2) = SIG3CCT(I+1)
            endif
         else
            SIGEND(1) = SIG3NC(I)
            SIGEND(2) = SIG3NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C	ENERGY FROM 100GEV TO 1000GEV  ( 10GEV/BIN )

      ELSE IF (E.LT.1000.0) THEN  
         ESTART = 100.
         WIDTH = 10.
         I = INT((E-ESTART)/WIDTH)
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG4CCE(I)
               SIGEND(2) = SIG4CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG4CCM(I)
               SIGEND(2) = SIG4CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG4CCT(I)
               SIGEND(2) = SIG4CCT(I+1)
            endif
         else
            SIGEND(1) = SIG4NC(I)
            SIGEND(2) = SIG4NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C       ENERGY OVER 1000GEV
      ELSE
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG4CCE(90)
               SIGEND(2) = SIG4CCE(90)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG4CCM(90)
               SIGEND(2) = SIG4CCM(90)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG4CCT(90)
               SIGEND(2) = SIG4CCT(90)
            endif
         else
            SIGEND(1) = SIG4NC(90)
            SIGEND(2) = SIG4NC(90)
         endif
         EEND(1) = 1000.
         EEND(2) = 1001.

      END IF

      SLOPE = (SIGEND(2)-SIGEND(1))/(EEND(2)-EEND(1))
      ScalingFactor = (float(NUMATOM)/12.)**(1./3.)
      COHCRSNEKART = (SIGEND(1)+SLOPE*(E-EEND(1)))*ScalingFactor

      RETURN
      END
