************************************************************************
*     -------------------------
      FUNCTION FNELSPAU(E,IPAR)
*     -------------------------
*
*     (Purpose)
*       TOTAL CROSS SECTION OF ELASTIC SCATTERING
*       FERMI MOMENTUM AND PAULI PRINCIPLE IS CONSIDERED.
*
*     (Input)
*       E      : NEUTRINO ENERGY ( GEV )
*       IPAR   : NEUTRINO TYPE
*                  12 : NEU-E
*                 -12 : NEU-E-BAR            
*                  14 : NEU-MU
*                 -14 : NEU-MU-BAR            
*                  16 : NEU-TAU
*                 -16 : NEU-TAU-BAR
*     
*     (Output)
*       FNELSPAU : CROSS SECTION ( 10**-38cm^2 )
*
*     (Creation Date and Author)
*       19??.??.?? ; M.Nakahata
*       1987.06.22 ; N.SATO     FOR TAU
*       1988.06.25 ; T.KAJITA   MA 0.95 -> 1.01
*       1988.06.28 ; M.TAKITA   UNIFIED
*       1995.02.16 ; K. KANEYUKI  FOR S.K.
*                    ELSPAU -> FNELSPAU
*                    TEMPORARY VERSION ( FNNUCL = 0. )
*                    CROSS SECTIONS OF NEU_TAU_BAR ARE LARGE AT
*                    HIGH ENERGY REGION, BUT WE ADD NO CORRECTION.
*       2000.09.05 ; K. KANEYUKI FIX FNNUCL, UPDATE CF
*     
C
C       2002.09.17 ; Y.Hayato ; 
C                      Cross-section was calculated with programs in ver 99b.
C                             modifications : dnels.F  
C                                                 line 65 was commented out
C                                                 line 57 do not subtract V
C                                                 fixed potential
C                                             dnelsq2.F
C                                                 MA=1.1 
C
C       2007.12.05 ; G.Mitsuka, MA=1.21 is added
C       2009.01.26 ; Y.Hayato   'Obsolete function'
C
************************************************************************
      INTEGER IFIRST
      DATA IFIRST/0/
      SAVE IFIRST

      LOGICAL LEXIST

      REAL EE(210) ,EE1(50) ,EE2(50) ,EE3(50) ,EE4(50), EE5(10)
      REAL EM(210) ,EM1(50) ,EM2(50) ,EM3(50) ,EM4(50), EM5(10)
      REAL ET(125) ,ET1(50) ,ET2(50) ,ET3(25)
  
      REAL CE(210) ,CE1(50) ,CE2(50) ,CE3(50) ,CE4(50) ,CE5(10)
      REAL CEB(210),CEB1(50),CEB2(50),CEB3(50) ,CEB4(50) ,CEB5(10)

      REAL CM(210) ,CM1(50) ,CM2(50) ,CM3(50) ,CM4(50) ,CM5(10)
      REAL CMB(210),CMB1(50),CMB2(50),CMB3(50) ,CMB4(50) ,CMB5(10)

      REAL CT(156) ,CT1(50) ,CT2(50) ,CT3(25)
      REAL CTB(156),CTB1(50),CTB2(50),CTB3(25)

      EQUIVALENCE (EE(1),EE1(1))
      EQUIVALENCE (EE(51),EE2(1))
      EQUIVALENCE (EE(101),EE3(1))
      EQUIVALENCE (EE(151),EE4(1))
      EQUIVALENCE (EE(201),EE5(1))

      EQUIVALENCE (EM(1),EM1(1))
      EQUIVALENCE (EM(51),EM2(1))
      EQUIVALENCE (EM(101),EM3(1))
      EQUIVALENCE (EM(151),EM4(1))
      EQUIVALENCE (EM(201),EM5(1))

      EQUIVALENCE (ET(1),ET1(1))
      EQUIVALENCE (ET(51),ET2(1))
      EQUIVALENCE (ET(101),ET3(1))

      EQUIVALENCE (CE(1),CE1(1))
      EQUIVALENCE (CE(51),CE2(1))
      EQUIVALENCE (CE(101),CE3(1))
      EQUIVALENCE (CE(151),CE4(1))
      EQUIVALENCE (CE(201),CE5(1))

      EQUIVALENCE (CM(1),CM1(1))
      EQUIVALENCE (CM(51),CM2(1))
      EQUIVALENCE (CM(101),CM3(1))
      EQUIVALENCE (CM(151),CM4(1))
      EQUIVALENCE (CM(201),CM5(1))

      EQUIVALENCE (CT(1),CT1(1))
      EQUIVALENCE (CT(51),CT2(1))
      EQUIVALENCE (CT(101),CT3(1))

      EQUIVALENCE (CEB(1),CEB1(1))
      EQUIVALENCE (CEB(51),CEB2(1))
      EQUIVALENCE (CEB(101),CEB3(1))
      EQUIVALENCE (CEB(151),CEB4(1))
      EQUIVALENCE (CEB(201),CEB5(1))

      EQUIVALENCE (CMB(1),CMB1(1))
      EQUIVALENCE (CMB(51),CMB2(1))
      EQUIVALENCE (CMB(101),CMB3(1))
      EQUIVALENCE (CMB(151),CMB4(1))
      EQUIVALENCE (CMB(201),CMB5(1))

      EQUIVALENCE (CTB(1),CTB1(1))
      EQUIVALENCE (CTB(51),CTB2(1))
      EQUIVALENCE (CTB(101),CTB3(1))
C     
C --  ENERGY OF ELECTRON NEUTRINO 
C
C

       DATA EE1/0.025,0.075,0.125,0.175,0.225, 
     $          0.275,0.325,0.375,0.425,0.475, 
     $          0.525,0.575,0.625,0.675,0.725, 
     $          0.775,0.825,0.875,0.925,0.975, 
     $          1.025,1.075,1.125,1.175,1.225, 
     $          1.275,1.325,1.375,1.425,1.475, 
     $          1.525,1.575,1.625,1.675,1.725, 
     $          1.775,1.825,1.875,1.925,1.975, 
     $          2.025,2.075,2.125,2.175,2.225, 
     $          2.275,2.325,2.375,2.425,2.475/

      DATA EE2/ 2.525,2.575,2.625,2.675,2.725, 
     $          2.775,2.825,2.875,2.925,2.975, 
     $          3.025,3.075,3.125,3.175,3.225, 
     $          3.275,3.325,3.375,3.425,3.475, 
     $          3.525,3.575,3.625,3.675,3.725, 
     $          3.775,3.825,3.875,3.925,3.975, 
     $          4.025,4.075,4.125,4.175,4.225, 
     $          4.275,4.325,4.375,4.425,4.475, 
     $          4.525,4.575,4.625,4.675,4.725, 
     $          4.775,4.825,4.875,4.925,4.975/

      DATA EE3/ 5.025,5.075,5.125,5.175,5.225, 
     $          5.275,5.325,5.375,5.425,5.475, 
     $          5.525,5.575,5.625,5.675,5.725, 
     $          5.775,5.825,5.875,5.925,5.975, 
     $          6.025,6.075,6.125,6.175,6.225, 
     $          6.275,6.325,6.375,6.425,6.475, 
     $          6.525,6.575,6.625,6.675,6.725, 
     $          6.775,6.825,6.875,6.925,6.975, 
     $          7.025,7.075,7.125,7.175,7.225, 
     $          7.275,7.325,7.375,7.425,7.475/
     
      DATA EE4/ 7.525,7.575,7.625,7.675,7.725, 
     $          7.775,7.825,7.875,7.925,7.975, 
     $          8.025,8.075,8.125,8.175,8.225, 
     $          8.275,8.325,8.375,8.425,8.475, 
     $          8.525,8.575,8.625,8.675,8.725, 
     $          8.775,8.825,8.875,8.925,8.975, 
     $          9.025,9.075,9.125,9.175,9.225, 
     $          9.275,9.325,9.375,9.425,9.475, 
     $          9.525,9.575,9.625,9.675,9.725, 
     $          9.775,9.825,9.875,9.925,9.975/

      DATA EE5/ 11.00,12.00,13.00,14.00,15.00,
     $          16.00,17.00,18.00,19.00,20.00/

C --  ENERGY OF MUON NEUTRINO
C
       DATA EM1/0.025,0.075,0.125,0.175,0.225, 
     $          0.275,0.325,0.375,0.425,0.475, 
     $          0.525,0.575,0.625,0.675,0.725, 
     $          0.775,0.825,0.875,0.925,0.975, 
     $          1.025,1.075,1.125,1.175,1.225, 
     $          1.275,1.325,1.375,1.425,1.475, 
     $          1.525,1.575,1.625,1.675,1.725, 
     $          1.775,1.825,1.875,1.925,1.975, 
     $          2.025,2.075,2.125,2.175,2.225, 
     $          2.275,2.325,2.375,2.425,2.475/

      DATA EM2/ 2.525,2.575,2.625,2.675,2.725, 
     $          2.775,2.825,2.875,2.925,2.975, 
     $          3.025,3.075,3.125,3.175,3.225, 
     $          3.275,3.325,3.375,3.425,3.475, 
     $          3.525,3.575,3.625,3.675,3.725, 
     $          3.775,3.825,3.875,3.925,3.975, 
     $          4.025,4.075,4.125,4.175,4.225, 
     $          4.275,4.325,4.375,4.425,4.475, 
     $          4.525,4.575,4.625,4.675,4.725, 
     $          4.775,4.825,4.875,4.925,4.975/

      DATA EM3/ 5.025,5.075,5.125,5.175,5.225, 
     $          5.275,5.325,5.375,5.425,5.475, 
     $          5.525,5.575,5.625,5.675,5.725, 
     $          5.775,5.825,5.875,5.925,5.975, 
     $          6.025,6.075,6.125,6.175,6.225, 
     $          6.275,6.325,6.375,6.425,6.475, 
     $          6.525,6.575,6.625,6.675,6.725, 
     $          6.775,6.825,6.875,6.925,6.975, 
     $          7.025,7.075,7.125,7.175,7.225, 
     $          7.275,7.325,7.375,7.425,7.475/
     
      DATA EM4/ 7.525,7.575,7.625,7.675,7.725, 
     $          7.775,7.825,7.875,7.925,7.975, 
     $          8.025,8.075,8.125,8.175,8.225, 
     $          8.275,8.325,8.375,8.425,8.475, 
     $          8.525,8.575,8.625,8.675,8.725, 
     $          8.775,8.825,8.875,8.925,8.975, 
     $          9.025,9.075,9.125,9.175,9.225, 
     $          9.275,9.325,9.375,9.425,9.475, 
     $          9.525,9.575,9.625,9.675,9.725, 
     $          9.775,9.825,9.875,9.925,9.975/

      DATA EM5/ 11.00,12.00,13.00,14.00,15.00,
     $          16.00,17.00,18.00,19.00,20.00/

C
C --  ENERGY OF TAU NEUTRINO
C      
      DATA ET1 /.270E+01,.280E+01,.290E+01,.300E+01,.310E+01,
     &          .320E+01,.330E+01,.340E+01,.350E+01,.360E+01,
     &          .370E+01,.380E+01,.390E+01,.400E+01,.410E+01,
     &          .420E+01,.430E+01,.440E+01,.450E+01,.460E+01,
     &          .470E+01,.480E+01,.490E+01,.500E+01,.510E+01,
     &          .520E+01,.530E+01,.540E+01,.550E+01,.560E+01,
     &          .570E+01,.580E+01,.590E+01,.600E+01,.610E+01,
     &          .620E+01,.630E+01,.640E+01,.650E+01,.660E+01,
     &          .670E+01,.680E+01,.690E+01,.700E+01,.710E+01,
     &          .720E+01,.730E+01,.740E+01,.750E+01,.760E+01/
      DATA ET2 /.770E+01,.780E+01,.790E+01,.800E+01,.810E+01,
     &          .820E+01,.830E+01,.840E+01,.850E+01,.860E+01,
     &          .870E+01,.880E+01,.890E+01,.900E+01,.910E+01,
     &          .920E+01,.930E+01,.940E+01,.950E+01,.960E+01,
     &          .970E+01,.980E+01,.990E+01,.100E+02,.100E+02,
     &          .110E+02,.120E+02,.130E+02,.140E+02,.150E+02,
     &          .160E+02,.170E+02,.180E+02,.190E+02,.200E+02,
     &          .210E+02,.220E+02,.230E+02,.240E+02,.250E+02,
     &          .260E+02,.270E+02,.280E+02,.290E+02,.300E+02,
     &          .310E+02,.320E+02,.330E+02,.340E+02,.350E+02/
      DATA ET3 /.360E+02,.370E+02,.380E+02,.390E+02,.400E+02,
     &          .410E+02,.420E+02,.430E+02,.440E+02,.450E+02,
     &          .460E+02,.470E+02,.480E+02,.490E+02,.500E+02,
     &          .510E+02,.520E+02,.530E+02,.540E+02,.550E+02,
     &          .560E+02,.570E+02,.580E+02,.590E+02,.600E+02/

#ifdef QEMA111
C
C----------Based on Smith-Moniz formula / Code by H.Nakamura
C PARAMETERS                                                             
C   FILENAME                                                             
C      Filename=                                                         
C crssects.MA=1.11.FLAT.PF=0.225.V=-0.027.nue                            
C   Potential related                                                    
C      NUCPOTVAR(FLAG)   =  -1                                           
C      NUCPOTFIX(FLAG)   =  1                                            
C      VINUCPOT (GeV)    =   -2.70000E-02                                
C      VFNUCPOT (GeV)    =  0.                                           
C      PFNUCPOT (GeV/c)  =   0.225000                                    
C   AVMASS related                                                       
C      DAVMASS  (GeV/c^2)=    1.1100000143051                            
C   Neutrino flavor                                                      
C      INUFLV            =  12                                           
C== 217MeV     
C== IPAR:        12
      DATA CE1 /0.000000,0.003258,0.028388,0.095283,0.204087,
     $          0.325756,0.440220,0.543357,0.633586,0.710967,
     $          0.776331,0.831081,0.876512,0.913714,0.944818,
     $          0.969783,0.990754,1.007366,1.020825,1.032154,
     $          1.040580,1.047158,1.052938,1.057395,1.060446,
     $          1.062596,1.063827,1.064645,1.065475,1.066031,
     $          1.065452,1.064926,1.064057,1.063297,1.062243,
     $          1.060963,1.059648,1.058437,1.057009,1.055472,
     $          1.053801,1.052450,1.051036,1.049486,1.047903,
     $          1.046269,1.044604,1.043247,1.041886,1.040470/
C--
      DATA CE2 /1.038968,1.037488,1.035972,1.034416,1.032974,
     $          1.031789,1.030612,1.029436,1.028214,1.026905,
     $          1.025447,1.024097,1.022970,1.021928,1.020894,
     $          1.019853,1.018756,1.017663,1.016585,1.015507,
     $          1.014422,1.013291,1.012283,1.011458,1.010650,
     $          1.009852,1.009055,1.008245,1.007382,1.006512,
     $          1.005656,1.004805,1.003960,1.003120,1.002267,
     $          1.001389,1.000486,0.999882,0.999301,0.998732,
     $          0.998175,0.997625,0.997069,0.996505,0.995923,
     $          0.995295,0.994645,0.994010,0.993384,0.992765/
C--
      DATA CE3 /0.992209,0.991654,0.991113,0.990579,0.990051,
     $          0.989538,0.989027,0.988528,0.988037,0.987552,
     $          0.987079,0.986609,0.986152,0.985700,0.985252,
     $          0.984816,0.984381,0.983959,0.983541,0.983127,
     $          0.982724,0.982322,0.981931,0.981543,0.981159,
     $          0.980786,0.980413,0.980050,0.979691,0.979334,
     $          0.978986,0.978639,0.978302,0.977966,0.977634,
     $          0.977310,0.976986,0.976672,0.976359,0.976048,
     $          0.975746,0.975444,0.975149,0.974855,0.974564,
     $          0.974281,0.973998,0.973723,0.973449,0.973176/
C--
      DATA CE4 /0.972911,0.972646,0.972387,0.972130,0.971874,
     $          0.971624,0.971374,0.971131,0.970889,0.970648,
     $          0.970412,0.970179,0.969951,0.969723,0.969498,
     $          0.969273,0.969053,0.968838,0.968623,0.968410,
     $          0.968198,0.967991,0.967787,0.967584,0.967383,
     $          0.967183,0.966986,0.966794,0.966602,0.966411,
     $          0.966222,0.966036,0.965853,0.965671,0.965491,
     $          0.965311,0.965134,0.964961,0.964788,0.964617,
     $          0.964446,0.964278,0.964114,0.963950,0.963787,
     $          0.963625,0.963465,0.963309,0.963153,0.962998/
C--
      DATA CE5 /0.960126,0.957789,0.955808,0.954107,0.952630,
     $          0.951337,0.950195,0.949180,0.948270,0.947452/
C--
C== 217MeV     
C== IPAR:       -12
      DATA CEB1/0.000000,0.001988,0.011723,0.028495,0.048713,
     $          0.069737,0.090884,0.112078,0.133184,0.154092,
     $          0.174628,0.194754,0.214297,0.233004,0.251471,
     $          0.268904,0.286131,0.302356,0.317950,0.333345,
     $          0.347534,0.361164,0.374781,0.387786,0.399992,
     $          0.411686,0.422735,0.433484,0.444211,0.454680,
     $          0.464096,0.473415,0.482289,0.491090,0.499459,
     $          0.507440,0.515193,0.522840,0.530091,0.537042,
     $          0.543673,0.550387,0.556846,0.562988,0.568911,
     $          0.574599,0.580076,0.585650,0.591048,0.596226/
C--
      DATA CEB2/0.601161,0.605957,0.610563,0.614981,0.619356,
     $          0.623826,0.628165,0.632374,0.636410,0.640240,
     $          0.643810,0.647364,0.651010,0.654623,0.658138,
     $          0.661543,0.664796,0.667956,0.671035,0.674023,
     $          0.676916,0.679679,0.682474,0.685359,0.688184,
     $          0.690940,0.693625,0.696227,0.698708,0.701116,
     $          0.703471,0.705767,0.708007,0.710191,0.712304,
     $          0.714338,0.716292,0.718475,0.720628,0.722742,
     $          0.724818,0.726852,0.728834,0.730764,0.732631,
     $          0.734410,0.736127,0.737817,0.739475,0.741102/
C--
      DATA CEB3/0.742751,0.744363,0.745953,0.747513,0.749045,
     $          0.750557,0.752039,0.753500,0.754938,0.756350,
     $          0.757745,0.759113,0.760464,0.761793,0.763098,
     $          0.764389,0.765655,0.766907,0.768138,0.769349,
     $          0.770547,0.771722,0.772885,0.774030,0.775156,
     $          0.776270,0.777365,0.778448,0.779515,0.780564,
     $          0.781603,0.782624,0.783636,0.784632,0.785612,
     $          0.786584,0.787539,0.788485,0.789418,0.790336,
     $          0.791246,0.792141,0.793028,0.793900,0.794762,
     $          0.795617,0.796457,0.797291,0.798113,0.798923/
C--
      DATA CEB4/0.799727,0.800518,0.801303,0.802077,0.802840,
     $          0.803596,0.804341,0.805081,0.805812,0.806531,
     $          0.807245,0.807950,0.808650,0.809339,0.810021,
     $          0.810694,0.811361,0.812023,0.812675,0.813321,
     $          0.813958,0.814590,0.815217,0.815835,0.816447,
     $          0.817051,0.817650,0.818245,0.818832,0.819413,
     $          0.819987,0.820556,0.821121,0.821679,0.822231,
     $          0.822776,0.823317,0.823855,0.824386,0.824911,
     $          0.825430,0.825945,0.826457,0.826963,0.827464,
     $          0.827958,0.828449,0.828938,0.829420,0.829897/
C--
      DATA CEB5/0.838793,0.846094,0.852336,0.857732,0.862443,
     $          0.866592,0.870274,0.873563,0.876518,0.879189/
C--
C== 217MeV     
C== IPAR:        14
      DATA CM1 /0.000000,0.000000,0.000000,0.035748,0.137993,
     $          0.264844,0.386320,0.496217,0.592630,0.675444,
     $          0.745388,0.804072,0.852815,0.893110,0.926254,
     $          0.953348,0.975870,0.993710,1.008907,1.020711,
     $          1.030189,1.038278,1.044301,1.048578,1.052510,
     $          1.055700,1.057581,1.058748,1.059280,1.059415,
     $          1.059967,1.059587,1.059219,1.058497,1.057478,
     $          1.056624,1.055524,1.054250,1.052911,1.051718,
     $          1.050366,1.048901,1.047302,1.045996,1.044685,
     $          1.043247,1.041748,1.040230,1.038635,1.037393/
C--
      DATA CM2 /1.036138,1.034834,1.033438,1.032054,1.030670,
     $          1.029221,1.027846,1.026752,1.025668,1.024585,
     $          1.023305,1.022100,1.020891,1.019641,1.018569,
     $          1.017615,1.016664,1.015698,1.014687,1.013664,
     $          1.012659,1.011661,1.010649,1.009604,1.008625,
     $          1.007859,1.007114,1.006373,1.005638,1.004885,
     $          1.004086,1.003275,1.002475,1.001683,1.000891,
     $          1.000095,0.999295,0.998468,0.997584,0.997024,
     $          0.996483,0.995953,0.995435,0.994924,0.994418,
     $          0.993894,0.993359,0.992776,0.992161,0.991556/
C--
      DATA CM3 /0.990973,0.990431,0.989903,0.989382,0.988867,
     $          0.988365,0.987865,0.987379,0.986900,0.986426,
     $          0.985963,0.985505,0.985057,0.984615,0.984178,
     $          0.983748,0.983324,0.982911,0.982502,0.982098,
     $          0.981703,0.981310,0.980928,0.980549,0.980174,
     $          0.979808,0.979444,0.979089,0.978737,0.978388,
     $          0.978048,0.977709,0.977378,0.977051,0.976726,
     $          0.976409,0.976093,0.975784,0.975479,0.975175,
     $          0.974879,0.974581,0.974293,0.974007,0.973723,
     $          0.973446,0.973170,0.972899,0.972631,0.972366/
C--
      DATA CM4 /0.972106,0.971846,0.971593,0.971341,0.971087,
     $          0.970843,0.970599,0.970361,0.970124,0.969889,
     $          0.969658,0.969430,0.969205,0.968982,0.968762,
     $          0.968542,0.968327,0.968115,0.967905,0.967697,
     $          0.967490,0.967287,0.967087,0.966888,0.966691,
     $          0.966495,0.966303,0.966114,0.965926,0.965739,
     $          0.965554,0.965371,0.965192,0.965014,0.964837,
     $          0.964661,0.964488,0.964319,0.964149,0.963982,
     $          0.963814,0.963650,0.963489,0.963328,0.963169,
     $          0.963009,0.962853,0.962700,0.962547,0.962395/
C--
      DATA CM5 /0.959581,0.957289,0.955348,0.953680,0.952233,
     $          0.950966,0.949846,0.948850,0.947959,0.947156/
C--
C== 217MeV     
C== IPAR:       -14
      DATA CMB1/0.000000,0.000000,0.000000,0.013963,0.038262,
     $          0.061508,0.083891,0.105817,0.127427,0.148708,
     $          0.169487,0.189833,0.209580,0.228684,0.247104,
     $          0.264781,0.282064,0.298248,0.314369,0.329407,
     $          0.343842,0.358123,0.371484,0.383934,0.396518,
     $          0.408755,0.420053,0.430852,0.441140,0.451053,
     $          0.461249,0.470537,0.479704,0.488413,0.496697,
     $          0.504946,0.512793,0.520299,0.527549,0.534733,
     $          0.541581,0.548132,0.554369,0.560679,0.566797,
     $          0.572615,0.578199,0.583591,0.588741,0.594043/
C--
      DATA CMB2/0.599171,0.604096,0.608784,0.613334,0.617741,
     $          0.621945,0.626080,0.630341,0.634484,0.638505,
     $          0.642223,0.645893,0.649448,0.652856,0.656321,
     $          0.659794,0.663169,0.666434,0.669564,0.672593,
     $          0.675549,0.678428,0.681210,0.683881,0.686535,
     $          0.689313,0.692039,0.694696,0.697291,0.699802,
     $          0.702204,0.704531,0.706808,0.709031,0.711197,
     $          0.713302,0.715348,0.717315,0.719176,0.721292,
     $          0.723378,0.725427,0.727441,0.729416,0.731352,
     $          0.733227,0.735049,0.736785,0.738451,0.740086/
C--
      DATA CMB3/0.741706,0.743328,0.744926,0.746495,0.748037,
     $          0.749557,0.751046,0.752517,0.753963,0.755384,
     $          0.756786,0.758162,0.759521,0.760857,0.762171,
     $          0.763464,0.764738,0.765997,0.767236,0.768453,
     $          0.769658,0.770840,0.772010,0.773160,0.774293,
     $          0.775414,0.776514,0.777604,0.778676,0.779732,
     $          0.780777,0.781804,0.782821,0.783823,0.784810,
     $          0.785786,0.786747,0.787699,0.788636,0.789560,
     $          0.790475,0.791373,0.792265,0.793145,0.794012,
     $          0.794871,0.795717,0.796555,0.797382,0.798197/
C--
      DATA CMB4/0.799005,0.799801,0.800590,0.801368,0.802132,
     $          0.802894,0.803644,0.804388,0.805122,0.805846,
     $          0.806564,0.807273,0.807976,0.808669,0.809355,
     $          0.810032,0.810703,0.811368,0.812024,0.812673,
     $          0.813314,0.813950,0.814580,0.815202,0.815817,
     $          0.816425,0.817028,0.817625,0.818216,0.818800,
     $          0.819377,0.819949,0.820517,0.821078,0.821633,
     $          0.822181,0.822726,0.823266,0.823800,0.824328,
     $          0.824850,0.825368,0.825883,0.826391,0.826895,
     $          0.827392,0.827886,0.828377,0.828862,0.829342/
C--
      DATA CMB5/0.838286,0.845627,0.851904,0.857330,0.862067,
     $          0.866239,0.869941,0.873248,0.876219,0.878905/
C--
C== 217MeV     
C== IPAR:        16
      DATA CT1 /0.000000,0.000000,0.000530,0.004253,0.011998,
     $          0.023556,0.038400,0.055922,0.075511,0.096601,
     $          0.118683,0.141312,0.164105,0.186733,0.208921,
     $          0.230429,0.251067,0.270668,0.289105,0.306302,
     $          0.322273,0.337331,0.351653,0.365362,0.378475,
     $          0.391060,0.403159,0.414805,0.426028,0.436871,
     $          0.447334,0.457439,0.467203,0.476643,0.485773,
     $          0.494606,0.503155,0.511433,0.519450,0.527217,
     $          0.534745,0.542044,0.549121,0.555986,0.562646,
     $          0.569112,0.575390,0.581485,0.587406,0.593157/
C--
      DATA CT2 /0.598748,0.604186,0.609495,0.614660,0.619686,
     $          0.624580,0.629341,0.633982,0.638497,0.642899,
     $          0.647190,0.651370,0.655447,0.659419,0.663296,
     $          0.667080,0.670769,0.674372,0.677888,0.681321,
     $          0.684674,0.687949,0.691149,0.694275,0.694275,
     $          0.721959,0.744372,0.762815,0.778209,0.791201,
     $          0.802286,0.811825,0.820133,0.827496,0.834000,
     $          0.839780,0.844950,0.849595,0.853795,0.857603,
     $          0.861076,0.864248,0.867158,0.869837,0.872310,
     $          0.874601,0.863467,0.838734,0.840395,0.828957/
C--
      DATA CT3 /0.817557,0.818838,0.807384,0.796024,0.797029,
     $          0.771598,0.760336,0.761089,0.749903,0.738869,
     $          0.739474,0.728548,0.717785,0.707186,0.707642,
     $          0.697176,0.686880,0.673762,0.663576,0.653563,
     $          0.643723,0.643984,0.634298,0.624783,0.615437/
C--
C== 217MeV     
C== IPAR:       -16
      DATA CTB1/0.000000,0.000000,0.000046,0.000479,0.001647,
     $          0.003803,0.007112,0.011661,0.017472,0.024517,
     $          0.032729,0.042013,0.052259,0.063342,0.075131,
     $          0.087487,0.100282,0.113378,0.126653,0.140009,
     $          0.153334,0.166568,0.179678,0.192650,0.205418,
     $          0.217974,0.230303,0.242392,0.254234,0.265839,
     $          0.277188,0.288277,0.299109,0.309684,0.320004,
     $          0.330072,0.339892,0.349468,0.358805,0.367907,
     $          0.376781,0.385432,0.393865,0.402085,0.410099,
     $          0.417912,0.425531,0.432959,0.440203,0.447267/
C--
      DATA CTB2/0.454158,0.460883,0.467464,0.473888,0.480158,
     $          0.486282,0.492258,0.498097,0.503797,0.509368,
     $          0.514812,0.520130,0.525330,0.530410,0.535379,
     $          0.540239,0.544990,0.549640,0.554188,0.558639,
     $          0.562995,0.567259,0.571434,0.575521,0.575521,
     $          0.612103,0.642254,0.667464,0.688812,0.707078,
     $          0.722860,0.736610,0.748709,0.759504,0.769127,
     $          0.777757,0.785536,0.792583,0.798999,0.804858,
     $          0.810234,0.815178,0.819741,0.823966,0.827888,
     $          0.831538,0.821842,0.798616,0.801404,0.791192/
C--
      DATA CTB3/0.780959,0.783189,0.772797,0.762452,0.764265,
     $          0.739876,0.729502,0.730948,0.720582,0.710335,
     $          0.711542,0.701345,0.691284,0.681362,0.682320,
     $          0.672485,0.662799,0.650200,0.640582,0.631120,
     $          0.621811,0.622448,0.613261,0.604230,0.595353/

#elif QEMA121
C   Potential related                                                    
C      NUCPOTVAR(FLAG)   = -1                                           
C      NUCPOTFIX(FLAG)   =  1                                            
C      VINUCPOT (GeV)    = -2.70000E-02
C      VFNUCPOT (GeV)    =  0.                                           
C      PFNUCPOT (GeV/c)  =  0.225
C   AVMASS related                                                       
C      DAVMASS  (GeV/c^2)=  1.21

C== IPAR:        12
      DATA CE1 /0.0000000, 0.0032664, 0.0286162, 0.0967627, 0.2089310,
     &  0.3358737, 0.4567218, 0.5669253, 0.6645057, 0.7492009,
     &  0.8216110, 0.8829919, 0.9345530, 0.9773431, 1.0135058,
     &  1.0429958, 1.0680141, 1.0882090, 1.1048399, 1.1189737,
     &  1.1298615, 1.1385971, 1.1462842, 1.1524143, 1.1569227,
     &  1.1603450, 1.1626818, 1.1644679, 1.1661516, 1.1674482,
     &  1.1674966, 1.1675205, 1.1671244, 1.1667738, 1.1660682,
     &  1.1650825, 1.1640159, 1.1630158, 1.1617596, 1.1603616,
     &  1.1587989, 1.1575376, 1.1561902, 1.1546844, 1.1531286,
     &  1.1515046, 1.1498356, 1.1484689, 1.1470861, 1.1456375/
C--
      DATA CE2 /1.1440936, 1.1425638, 1.1409913, 1.1393723, 1.1378644,
     &  1.1366130, 1.1353652, 1.1341153, 1.1328152, 1.1314251,
     &  1.1298798, 1.1284437, 1.1272317, 1.1261051, 1.1249852,
     &  1.1238573, 1.1226718, 1.1214899, 1.1203227, 1.1191549,
     &  1.1179798, 1.1167585, 1.1156622, 1.1147513, 1.1138589,
     &  1.1129758, 1.1120943, 1.1112006, 1.1102527, 1.1092989,
     &  1.1083593, 1.1074256, 1.1064984, 1.1055764, 1.1046419,
     &  1.1036830, 1.1026999, 1.1020209, 1.1013658, 1.1007241,
     &  1.1000938, 1.0994722, 1.0988452, 1.0982118, 1.0975600,
     &  1.0968614, 1.0961429, 1.0954388, 1.0947447, 1.0940597/
C--
       DATA CE3 /1.0934378, 1.0928174, 1.0922134, 1.0916164, 1.0910262,
     &  1.0904518, 1.0898807, 1.0893222, 1.0887723, 1.0882282,
     &  1.0876986, 1.0871719, 1.0866588, 1.0861514, 1.0856488,
     &  1.0851592, 1.0846713, 1.0841968, 1.0837270, 1.0832617,
     &  1.0828083, 1.0823565, 1.0819162, 1.0814803, 1.0810483,
     &  1.0806277, 1.0802082, 1.0797992, 1.0793937, 1.0789918,
     &  1.0785998, 1.0782090, 1.0778280, 1.0774502, 1.0770749,
     &  1.0767096, 1.0763446, 1.0759891, 1.0756363, 1.0752857,
     &  1.0749443, 1.0746028, 1.0742702, 1.0739374, 1.0736090,
     &  1.0732895, 1.0729696, 1.0726579, 1.0723484, 1.0720403/
C--
       DATA CE4 /1.0717404, 1.0714400, 1.0711477, 1.0708567, 1.0705673,
     &  1.0702837, 1.0700009, 1.0697260, 1.0694523, 1.0691794,
     &  1.0689121, 1.0686480, 1.0683891, 1.0681310, 1.0678759,
     &  1.0676216, 1.0673723, 1.0671279, 1.0668843, 1.0666435,
     &  1.0664033, 1.0661677, 1.0659368, 1.0657065, 1.0654787,
     &  1.0652515, 1.0650287, 1.0648099, 1.0645921, 1.0643764,
     &  1.0641611, 1.0639499, 1.0637426, 1.0635358, 1.0633312,
     &  1.0631270, 1.0629267, 1.0627300, 1.0625337, 1.0623393,
     &  1.0621455, 1.0619551, 1.0617682, 1.0615816, 1.0613968,
     &  1.0612123, 1.0610312, 1.0608535, 1.0606759, 1.0605000/
C--
       DATA CE5 /1.0572354, 1.0545756, 1.0523198, 1.0503815, 1.0486985,
     &  1.0472239, 1.0459214, 1.0447626, 1.0437251, 1.0427908/

C== IPAR:       -12
       DATA CEB1/0.0000000, 0.0019938, 0.0118445, 0.0291021, 0.0502913,
     &  0.0725290, 0.0949256, 0.1173550, 0.1396648, 0.1617427,
     &  0.1834227, 0.2046751, 0.2253345, 0.2451497, 0.2647280,
     &  0.2832632, 0.3015995, 0.3189236, 0.3356137, 0.3521025,
     &  0.3673720, 0.3820671, 0.3967480, 0.4108044, 0.4240406,
     &  0.4367480, 0.4487917, 0.4605199, 0.4722177, 0.4836381,
     &  0.4939770, 0.5042047, 0.5139673, 0.5236425, 0.5328645,
     &  0.5416787, 0.5502476, 0.5586951, 0.5667267, 0.5744408,
     &  0.5818157, 0.5892617, 0.5964363, 0.6032755, 0.6098799,
     &  0.6162333, 0.6223595, 0.6285743, 0.6345972, 0.6403860/
C--
       DATA CEB2/0.6459174, 0.6512960, 0.6564710, 0.6614457, 0.6663663,
     &  0.6713735, 0.6762385, 0.6809610, 0.6855001, 0.6898209,
     &  0.6938692, 0.6978925, 0.7020003, 0.7060685, 0.7100282,
     &  0.7138690, 0.7175478, 0.7211248, 0.7246134, 0.7280021,
     &  0.7312874, 0.7344351, 0.7376089, 0.7408681, 0.7440593,
     &  0.7471762, 0.7502140, 0.7531621, 0.7559826, 0.7587231,
     &  0.7614047, 0.7640215, 0.7665762, 0.7690699, 0.7714867,
     &  0.7738177, 0.7760642, 0.7785388, 0.7809786, 0.7833748,
     &  0.7857273, 0.7880347, 0.7902847, 0.7924778, 0.7946038,
     &  0.7966365, 0.7986036, 0.8005387, 0.8024385, 0.8043035/
C--
       DATA CEB3/0.8061873, 0.8080299, 0.8098477, 0.8116326, 0.8133852,
     &  0.8151155, 0.8168120, 0.8184845, 0.8201303, 0.8217474,
     &  0.8233448, 0.8249123, 0.8264607, 0.8279832, 0.8294798,
     &  0.8309591, 0.8324105, 0.8338463, 0.8352587, 0.8366479,
     &  0.8380216, 0.8393707, 0.8407053, 0.8420189, 0.8433116,
     &  0.8445913, 0.8458483, 0.8470923, 0.8483171, 0.8495230,
     &  0.8507169, 0.8518905, 0.8530528, 0.8541975, 0.8553246,
     &  0.8564417, 0.8575398, 0.8586281, 0.8597003, 0.8607566,
     &  0.8618039, 0.8628336, 0.8638546, 0.8648584, 0.8658501,
     &  0.8668340, 0.8678016, 0.8687615, 0.8697079, 0.8706409/
C--
       DATA CEB4/0.8715667, 0.8724776, 0.8733818, 0.8742734, 0.8751525,
     &  0.8760237, 0.8768824, 0.8777357, 0.8785771, 0.8794067,
     &  0.8802292, 0.8810425, 0.8818488, 0.8826441, 0.8834305,
     &  0.8842064, 0.8849759, 0.8857389, 0.8864918, 0.8872366,
     &  0.8879715, 0.8887005, 0.8894239, 0.8901377, 0.8908439,
     &  0.8915411, 0.8922330, 0.8929193, 0.8935971, 0.8942679,
     &  0.8949301, 0.8955874, 0.8962396, 0.8968838, 0.8975216,
     &  0.8981514, 0.8987766, 0.8993974, 0.9000105, 0.9006176,
     &  0.9012174, 0.9018129, 0.9024043, 0.9029886, 0.9035673,
     &  0.9041389, 0.9047068, 0.9052709, 0.9058283, 0.9063805/
C--
       DATA CEB5/0.9166698, 0.9251239, 0.9323574, 0.9386151, 0.9440817,
     &  0.9488983, 0.9531742, 0.9569957, 0.9604313, 0.9635368/

C== IPAR:        14
       DATA CM1 /0.0000000, 0.0000000, 0.0000000, 0.0361515, 0.1408229,
     &  0.2724698, 0.4000976, 0.5169820, 0.6207757, 0.7110065,
     &  0.7881386, 0.8536177, 0.9086598, 0.9547160, 0.9930732,
     &  1.0248448, 1.0515488, 1.0730945, 1.0915850, 1.1062880,
     &  1.1183209, 1.1286653, 1.1366626, 1.1426659, 1.1481208,
     &  1.1526473, 1.1556909, 1.1578708, 1.1592817, 1.1601830,
     &  1.1614108, 1.1616096, 1.1617466, 1.1614580, 1.1608080,
     &  1.1602738, 1.1594427, 1.1583951, 1.1572438, 1.1562093,
     &  1.1549839, 1.1536177, 1.1520931, 1.1508466, 1.1495764,
     &  1.1481607, 1.1466697, 1.1451459, 1.1435329, 1.1422687/
C--
       DATA CM2 /1.1409829, 1.1396383, 1.1381940, 1.1367556, 1.1353115,
     &  1.1337969, 1.1323532, 1.1311910, 1.1300364, 1.1288795,
     &  1.1275198, 1.1262338, 1.1249429, 1.1236084, 1.1224527,
     &  1.1214174, 1.1203830, 1.1193334, 1.1182367, 1.1171286,
     &  1.1160371, 1.1149533, 1.1138546, 1.1127231, 1.1116587,
     &  1.1108109, 1.1099850, 1.1091630, 1.1083478, 1.1075145,
     &  1.1066349, 1.1057431, 1.1048640, 1.1039927, 1.1031222,
     &  1.1022484, 1.1013707, 1.1004666, 1.0995052, 1.0988733,
     &  1.0982618, 1.0976622, 1.0970759, 1.0964959, 1.0959225,
     &  1.0953313, 1.0947298, 1.0940803, 1.0933999, 1.0927293/
C--
       DATA CM3 /1.0920814, 1.0914763, 1.0908855, 1.0903025, 1.0897267,
     &  1.0891647, 1.0886053, 1.0880613, 1.0875238, 1.0869927,
     &  1.0864744, 1.0859600, 1.0854579, 1.0849617, 1.0844709,
     &  1.0839878, 1.0835121, 1.0830475, 1.0825878, 1.0821329,
     &  1.0816890, 1.0812474, 1.0808166, 1.0803899, 1.0799677,
     &  1.0795556, 1.0791453, 1.0787446, 1.0783478, 1.0779548,
     &  1.0775709, 1.0771891, 1.0768156, 1.0764463, 1.0760798,
     &  1.0757216, 1.0753651, 1.0750165, 1.0746709, 1.0743283,
     &  1.0739934, 1.0736576, 1.0733315, 1.0730079, 1.0726873,
     &  1.0723735, 1.0720611, 1.0717553, 1.0714520, 1.0711514/
C--
       DATA CM4 /1.0708570, 1.0705636, 1.0702764, 1.0699914, 1.0697047,
     &  1.0694280, 1.0691519, 1.0688818, 1.0686135, 1.0683471,
     &  1.0680852, 1.0678264, 1.0675720, 1.0673192, 1.0670692,
     &  1.0668206, 1.0665764, 1.0663362, 1.0660974, 1.0658615,
     &  1.0656270, 1.0653961, 1.0651691, 1.0649434, 1.0647202,
     &  1.0644981, 1.0642796, 1.0640649, 1.0638511, 1.0636398,
     &  1.0634289, 1.0632219, 1.0630186, 1.0628158, 1.0626153,
     &  1.0624152, 1.0622188, 1.0620258, 1.0618333, 1.0616428,
     &  1.0614529, 1.0612662, 1.0610827, 1.0608999, 1.0607187,
     &  1.0605381, 1.0603604, 1.0601860, 1.0600119, 1.0598395/
C--
       DATA CM5 /1.0566380, 1.0540284, 1.0518157, 1.0499145, 1.0482634,
     &  1.0468167, 1.0455388, 1.0444016, 1.0433835, 1.0424666/

C== IPAR:       -14
       DATA CMB1/0.0000000, 0.0000000, 0.0000000, 0.0141644, 0.0392996,
     &  0.0637787, 0.0874675, 0.1106879, 0.1335532, 0.1560469,
     &  0.1780030, 0.1995013, 0.2203832, 0.2406115, 0.2601500,
     &  0.2789389, 0.2973346, 0.3146202, 0.3318460, 0.3479753,
     &  0.3634922, 0.3788510, 0.3932714, 0.4067648, 0.4203840,
     &  0.4336436, 0.4459416, 0.4577215, 0.4689703, 0.4798269,
     &  0.4909597, 0.5011564, 0.5112170, 0.5207987, 0.5299356,
     &  0.5390223, 0.5476879, 0.5559926, 0.5640247, 0.5719752,
     &  0.5795716, 0.5868527, 0.5938025, 0.6008138, 0.6076168,
     &  0.6141039, 0.6203403, 0.6263702, 0.6321425, 0.6380579/
C--
       DATA CMB2/0.6437861, 0.6492967, 0.6545563, 0.6596659, 0.6646190,
     &  0.6693572, 0.6740159, 0.6787933, 0.6834421, 0.6879576,
     &  0.6921563, 0.6962990, 0.7003160, 0.7041750, 0.7080851,
     &  0.7119959, 0.7158000, 0.7194852, 0.7230254, 0.7264563,
     &  0.7298076, 0.7330732, 0.7362342, 0.7392764, 0.7422954,
     &  0.7454346, 0.7485149, 0.7515206, 0.7544571, 0.7573028,
     &  0.7600331, 0.7626820, 0.7652749, 0.7678092, 0.7702796,
     &  0.7726842, 0.7750243, 0.7772801, 0.7794235, 0.7818228,
     &  0.7841871, 0.7865095, 0.7887930, 0.7910318, 0.7932281,
     &  0.7953590, 0.7974333, 0.7994158, 0.8013241, 0.8031984/
C--
       DATA CMB3/0.8050531, 0.8069064, 0.8087330, 0.8105276, 0.8122907,
     &  0.8140293, 0.8157336, 0.8174169, 0.8190715, 0.8206979,
     &  0.8223031, 0.8238794, 0.8254357, 0.8269663, 0.8284716,
     &  0.8299544, 0.8314149, 0.8328579, 0.8342776, 0.8356744,
     &  0.8370551, 0.8384119, 0.8397534, 0.8410738, 0.8423740,
     &  0.8436598, 0.8449236, 0.8461739, 0.8474051, 0.8486179,
     &  0.8498177, 0.8509982, 0.8521660, 0.8533174, 0.8544515,
     &  0.8555738, 0.8566785, 0.8577720, 0.8588499, 0.8599124,
     &  0.8609647, 0.8619983, 0.8630242, 0.8640358, 0.8650336,
     &  0.8660219, 0.8669955, 0.8679599, 0.8689113, 0.8698502/
C--
       DATA CMB4/0.8707803, 0.8716967, 0.8726049, 0.8735012, 0.8743819,
     &  0.8752587, 0.8761230, 0.8769798, 0.8778256, 0.8786605,
     &  0.8794873, 0.8803049, 0.8811147, 0.8819141, 0.8827048,
     &  0.8834853, 0.8842589, 0.8850251, 0.8857819, 0.8865308,
     &  0.8872705, 0.8880033, 0.8887296, 0.8894472, 0.8901573,
     &  0.8908587, 0.8915541, 0.8922436, 0.8929248, 0.8935991,
     &  0.8942649, 0.8949257, 0.8955811, 0.8962286, 0.8968697,
     &  0.8975031, 0.8981315, 0.8987553, 0.8993715, 0.8999819,
     &  0.9005849, 0.9011835, 0.9017778, 0.9023651, 0.9029468,
     &  0.9035217, 0.9040925, 0.9046594, 0.9052196, 0.9057747/
C--
       DATA CMB5/0.9161174, 0.9246144, 0.9318855, 0.9381757, 0.9436706,
     &  0.9485123, 0.9528103, 0.9566514, 0.9601047, 0.9632261/

C== IPAR:        16
       DATA CT1 /0.0000000, 0.0000000, 0.0006177, 0.0049491, 0.0139391,
     &  0.0273232, 0.0444739, 0.0646696, 0.0871956, 0.1113898,
     &  0.1366614, 0.1624945, 0.1884476, 0.2141462, 0.2392762,
     &  0.2635666, 0.2868059, 0.3088075, 0.3294324, 0.3485969,
     &  0.3663270, 0.3829945, 0.3988085, 0.4139110, 0.4283300,
     &  0.4421462, 0.4554082, 0.4681554, 0.4804242, 0.4922620,
     &  0.5036740, 0.5146832, 0.5253116, 0.5355780, 0.5454993,
     &  0.5550905, 0.5643669, 0.5733433, 0.5820314, 0.5904433,
     &  0.5985920, 0.6064883, 0.6141409, 0.6215612, 0.6287567,
     &  0.6357386, 0.6425153, 0.6490923, 0.6554790, 0.6616808/
C--
       DATA CT2 /0.6677075, 0.6735666, 0.6792844, 0.6848459, 0.6902564,
     &  0.6955225, 0.7006453, 0.7056360, 0.7104919, 0.7152243,
     &  0.7198363, 0.7243279, 0.7287086, 0.7329757, 0.7371397,
     &  0.7412020, 0.7451634, 0.7490312, 0.7528040, 0.7564886,
     &  0.7600862, 0.7635994, 0.7670320, 0.7703850, 0.7703850,
     &  0.8000619, 0.8240671, 0.8438102, 0.8602818, 0.8741814,
     &  0.8860396, 0.8962455, 0.9051330, 0.9130070, 0.9199625,
     &  0.9261456, 0.9316748, 0.9366448, 0.9411381, 0.9452134,
     &  0.9489300, 0.9523256, 0.9554411, 0.9583098, 0.9609590,
     &  0.9634128, 0.9520930, 0.9268207, 0.9286090, 0.9169103/
C--
       DATA CT3 /0.9052220, 0.9066046, 0.8948309, 0.8831273, 0.8842150,
     &  0.8581067, 0.8464542, 0.8472736, 0.8356764, 0.8242139,
     &  0.8248747, 0.8135021, 0.8022790, 0.7912085, 0.7917075,
     &  0.7807560, 0.7699643, 0.7566041, 0.7459098, 0.7353808,
     &  0.7250171, 0.7253062, 0.7150872, 0.7050335, 0.6951440/

C== IPAR:       -16
        DATA CTB1/0.0000000, 0.0000000, 0.0000520, 0.0005378, 0.0018510,
     &  0.0042762, 0.0079957, 0.0131038, 0.0196218, 0.0275138,
     &  0.0367016, 0.0470766, 0.0585102, 0.0708618, 0.0839839,
     &  0.0977212, 0.1119282, 0.1264535, 0.1411607, 0.1559406,
     &  0.1706690, 0.1852824, 0.1997438, 0.2140382, 0.2280961,
     &  0.2419103, 0.2554644, 0.2687458, 0.2817483, 0.2944821,
     &  0.3069270, 0.3190822, 0.3309495, 0.3425307, 0.3538288,
     &  0.3648467, 0.3755895, 0.3860632, 0.3962721, 0.4062219,
     &  0.4159200, 0.4253729, 0.4345850, 0.4435644, 0.4523158,
     &  0.4608476, 0.4691663, 0.4772756, 0.4851832, 0.4928935/
C--
       DATA CTB2/0.5004149, 0.5077541, 0.5149349, 0.5219435, 0.5287847,
     &  0.5354646, 0.5419843, 0.5483543, 0.5545723, 0.5606490,
     &  0.5665877, 0.5723885, 0.5780605, 0.5836018, 0.5890223,
     &  0.5943236, 0.5995071, 0.6045798, 0.6095407, 0.6143968,
     &  0.6191494, 0.6238013, 0.6283566, 0.6328166, 0.6328166,
     &  0.6727447, 0.7056736, 0.7332265, 0.7565751, 0.7765687,
     &  0.7938591, 0.8089350, 0.8222108, 0.8340569, 0.8446254,
     &  0.8541081, 0.8626624, 0.8704154, 0.8774773, 0.8839307,
     &  0.8898548, 0.8953053, 0.9003380, 0.9049998, 0.9093295,
     &  0.9133614, 0.9036890, 0.8801280, 0.8832254, 0.8729307/
C--
       DATA CTB3/0.8625795, 0.8650660, 0.8545074, 0.8439652, 0.8459944,
     &  0.8210639, 0.8104282, 0.8120563, 0.8013957, 0.7908334,
     &  0.7921968, 0.7816577, 0.7712376, 0.7609409, 0.7620280,
     &  0.7517974, 0.7417018, 0.7289401, 0.7188954, 0.7089947,
     &  0.6992388, 0.6999695, 0.6903215, 0.6808210, 0.6714675/

#elif QEMA131
C-- nothing here because this part is obsolete.
#else
#error You must set MA in Imakefile
#endif

C--
C --  Message
C--
       write(*,*) 'Obsolete.'
       STOP

       if (ifirst.eq.0) then
          ifirst = 1
#ifdef QEMA111
          write(*,*) 'FNELSPAU: MA for QE = 1.11'
          inquire(file='ccqe_xsec_ma1.1.dat',exist=lexist)
          if (lexist.eqv..false.) then
             write(*,*) 'FNELSPAU: Table for QE MA = 1.11 is missing.'
             stop
          endif
#elif  QEMA121
          write(*,*) 'FNELSPAU: MA for QE = 1.21'
          inquire(file='ccqe_xsec_ma1.2.dat',exist=lexist)
          if (lexist.eqv..false.) then
             write(*,*) 'FNELSPAU: Table for QE MA = 1.21 is missing.'
             stop
          endif
#elif  QEMA131
          write(*,*) 'FNELSPAU: MA for QE = 1.31'
          inquire(file='ccqe_xsec_ma1.3.dat',exist=lexist)
          if (lexist.eqv..false.) then
             write(*,*) 'FNELSPAU: Table for QE MA = 1.31 is missing.'
             stop
          endif
#endif
       endif
C--
C --  CHECK PARTICLE SPECIES
C      
      FNELSPAU=0.
C
C ++ FOR E-NEUTRINO
C
      IF (IABS(IPAR).EQ.12) THEN
         IF(E.LE.EE(1))RETURN
         DO 10 I=2,210
            IF (EE(I).GT.E) GOTO 20
 10      CONTINUE
         IF (IPAR.EQ.12)  FNELSPAU=CE(210)
         IF (IPAR.EQ.-12) FNELSPAU=CEB(210)
         RETURN
 20      CONTINUE

         IF (IPAR.EQ.12) THEN
            FNELSPAU=CE(I-1)+(CE(I)-CE(I-1))/(EE(I)-EE(I-1))
     &           *(E-EE(I-1))
         ELSE   
            FNELSPAU=CEB(I-1)+(CEB(I)-CEB(I-1))/(EE(I)-EE(I-1))
     &           *(E-EE(I-1))
         ENDIF
         RETURN
C
C ++ FOR MU-NEUTRINO
C
      ELSE IF (IABS(IPAR).EQ.14) THEN
         IF(E.LE.EE(1))RETURN
         DO 100 I=2,210
            IF (EM(I).GT.E) GOTO 110
 100     CONTINUE
         IF (IPAR.EQ. 14) FNELSPAU=CM(210)
         IF (IPAR.EQ.-14) FNELSPAU=CMB(210)
         RETURN
 110     CONTINUE

         IF (IPAR.EQ.14) THEN
            FNELSPAU=CM(I-1)+(CM(I)-CM(I-1))/(EM(I)-EM(I-1))
     &           *(E-EM(I-1))
         ELSE   
            FNELSPAU=CMB(I-1)+(CMB(I)-CMB(I-1))/(EM(I)-EM(I-1))
     &           *(E-EM(I-1))
         ENDIF
         RETURN
C
C ++ FOR TAU-NEUTRINO
C
      ELSE IF (IABS(IPAR).EQ.16) THEN
         IF(E.LE.2.7)RETURN
         DO 200 I=2,125
            IF (ET(I).GT.E) GOTO 210
 200     CONTINUE
         IF (IPAR.EQ. 16) FNELSPAU=CT(125)
         IF (IPAR.EQ.-16) FNELSPAU=CTB(125)
         RETURN
 210     CONTINUE

         IF (IPAR.EQ.16) THEN
            FNELSPAU=CT(I-1)+(CT(I)-CT(I-1))/(ET(I)-ET(I-1))
     &           *(E-ET(I-1))
         ELSE   
            FNELSPAU=CTB(I-1)+(CTB(I)-CTB(I-1))/(ET(I)-ET(I-1))
     &           *(E-ET(I-1))
         ENDIF
         RETURN
      ENDIF
      RETURN
      END
